@@ -237,6 +237,12 @@ select_devices (CreateCall *call)
   g_variant_builder_init (&options, G_VARIANT_TYPE_VARDICT);
   g_variant_builder_add (&options, "{sv}", "handle_token", g_variant_new_string (token));
   g_variant_builder_add (&options, "{sv}", "types", g_variant_new_uint32 (call->devices));
+  if (call->portal->remote_desktop_interface_version >= 2)
+    {
+      g_variant_builder_add (&options, "{sv}", "persist_mode", g_variant_new_uint32 (call->persist_mode));
+      if (call->restore_token)
+        g_variant_builder_add (&options, "{sv}", "restore_token", g_variant_new_string (call->restore_token));
+    }
   g_dbus_connection_call (call->portal->bus,
                           PORTAL_BUS_NAME,
                           PORTAL_OBJECT_PATH,
@@ -356,47 +362,6 @@ create_session (CreateCall *call)
                           call);
 }
 
-static void
-get_remote_desktop_interface_version_returned (GObject *object,
-                                               GAsyncResult *result,
-                                               gpointer data)
-{
-  CreateCall *call = data;
-  GError *error = NULL;
-  g_autoptr(GVariant) version_variant = NULL;
-  g_autoptr(GVariant) ret = NULL;
-
-  ret = g_dbus_connection_call_finish (G_DBUS_CONNECTION (object), result, &error);
-  if (error)
-    {
-      g_task_return_error (call->task, error);
-      create_call_free (call);
-      return;
-    }
-
-  g_variant_get_child (ret, 0, "v", &version_variant);
-  call->portal->remote_desktop_interface_version = g_variant_get_uint32 (version_variant);
-
-  create_session (call);
-}
-
-static void
-get_remote_desktop_interface_version (CreateCall *call)
-{
-  g_dbus_connection_call (call->portal->bus,
-                          PORTAL_BUS_NAME,
-                          PORTAL_OBJECT_PATH,
-                          "org.freedesktop.DBus.Properties",
-                          "Get",
-                          g_variant_new ("(ss)", "org.freedesktop.portal.RemoteDesktop", "version"),
-                          NULL,
-                          G_DBUS_CALL_FLAGS_NONE,
-                          -1,
-                          g_task_get_cancellable (call->task),
-                          get_remote_desktop_interface_version_returned,
-                          call);
-}
-
 static void
 get_screencast_interface_version_returned (GObject *object,
                                            GAsyncResult *result,
@@ -418,7 +383,7 @@ get_screencast_interface_version_returned (GObject *object,
   g_variant_get_child (ret, 0, "v", &version_variant);
   call->portal->screencast_interface_version = g_variant_get_uint32 (version_variant);
 
-  get_remote_desktop_interface_version (call);
+  create_session (call);
 }
 
 static void
@@ -439,19 +404,49 @@ get_screencast_interface_version (CreateCall *call)
 }
 
 static void
-get_portal_interface_versions (CreateCall *call)
+get_remote_desktop_interface_version_returned (GObject *object,
+                                               GAsyncResult *result,
+                                               gpointer data)
 {
-  g_assert (call != NULL);
-  g_assert (call->portal != NULL);
+  CreateCall *call = data;
+  GError *error = NULL;
+  g_autoptr(GVariant) version_variant = NULL;
+  g_autoptr(GVariant) ret = NULL;
+
+  ret = g_dbus_connection_call_finish (G_DBUS_CONNECTION (object), result, &error);
+  if (error)
+    {
+      g_task_return_error (call->task, error);
+      create_call_free (call);
+      return;
+    }
+
+  g_variant_get_child (ret, 0, "v", &version_variant);
+  call->portal->remote_desktop_interface_version = g_variant_get_uint32 (version_variant);
 
   if (call->portal->screencast_interface_version == 0)
     get_screencast_interface_version (call);
-  else if (call->portal->remote_desktop_interface_version == 0)
-    get_remote_desktop_interface_version (call);
   else
     create_session (call);
 }
 
+static void
+get_remote_desktop_interface_version (CreateCall *call)
+{
+  g_dbus_connection_call (call->portal->bus,
+                          PORTAL_BUS_NAME,
+                          PORTAL_OBJECT_PATH,
+                          "org.freedesktop.DBus.Properties",
+                          "Get",
+                          g_variant_new ("(ss)", "org.freedesktop.portal.RemoteDesktop", "version"),
+                          NULL,
+                          G_DBUS_CALL_FLAGS_NONE,
+                          -1,
+                          g_task_get_cancellable (call->task),
+                          get_remote_desktop_interface_version_returned,
+                          call);
+}
+
 /**
  * xdp_portal_create_screencast_session:
  * @portal: a [class@Portal]
@@ -496,7 +491,10 @@ xdp_portal_create_screencast_session (XdpPortal *portal,
   call->multiple = (flags & XDP_SCREENCAST_FLAG_MULTIPLE) != 0;
   call->task = g_task_new (portal, cancellable, callback, data);
 
-  get_portal_interface_versions (call);
+  if (portal->screencast_interface_version == 0)
+    get_screencast_interface_version (call);
+  else
+    create_session (call);
 }
 
 /**
@@ -551,6 +549,48 @@ xdp_portal_create_remote_desktop_session (XdpPortal *portal,
                                           GCancellable *cancellable,
                                           GAsyncReadyCallback  callback,
                                           gpointer data)
+{
+  xdp_portal_create_remote_desktop_session_full (portal,
+                                                 devices,
+                                                 outputs,
+                                                 flags,
+                                                 cursor_mode,
+                                                 XDP_PERSIST_MODE_NONE,
+                                                 NULL,
+                                                 cancellable,
+                                                 callback,
+                                                 data);
+}
+
+/**
+ * xdp_portal_create_remote_desktop_session_full: (finish-func xdp_portal_create_remote_desktop_session_finish)
+ * @portal: a [class@Portal]
+ * @devices: which kinds of input devices to ofer in the new dialog
+ * @outputs: which kinds of source to offer in the dialog
+ * @flags: options for this call
+ * @cursor_mode: the cursor mode of the session
+ * @persist_mode: the persist mode of the session
+ * @restore_token: (nullable): the token of a previous screencast session to restore
+ * @cancellable: (nullable): optional [class@Gio.Cancellable]
+ * @callback: (scope async): a callback to call when the request is done
+ * @data: (closure): data to pass to @callback
+ *
+ * Creates a session for remote desktop.
+ *
+ * When the request is done, @callback will be called. You can then
+ * call [method@Portal.create_remote_desktop_session_finish] to get the results.
+ */
+void
+xdp_portal_create_remote_desktop_session_full (XdpPortal *portal,
+                                               XdpDeviceType devices,
+                                               XdpOutputType outputs,
+                                               XdpRemoteDesktopFlags flags,
+                                               XdpCursorMode cursor_mode,
+                                               XdpPersistMode persist_mode,
+                                               const char *restore_token,
+                                               GCancellable *cancellable,
+                                               GAsyncReadyCallback  callback,
+                                               gpointer data)
 {
   CreateCall *call;
 
@@ -563,12 +603,15 @@ xdp_portal_create_remote_desktop_session (XdpPortal *portal,
   call->devices = devices;
   call->outputs = outputs;
   call->cursor_mode = cursor_mode;
-  call->persist_mode = XDP_PERSIST_MODE_NONE;
-  call->restore_token = NULL;
+  call->persist_mode = persist_mode;
+  call->restore_token = g_strdup (restore_token);
   call->multiple = (flags & XDP_REMOTE_DESKTOP_FLAG_MULTIPLE) != 0;
   call->task = g_task_new (portal, cancellable, callback, data);
 
-  get_portal_interface_versions (call);
+  if (portal->remote_desktop_interface_version == 0)
+    get_remote_desktop_interface_version (call);
+  else
+    create_session (call);
 }
 
 /**
@@ -826,29 +869,6 @@ xdp_session_start_finish (XdpSession *session,
   return g_task_propagate_boolean (G_TASK (result), error);
 }
 
-/**
- * xdp_session_close:
- * @session: an active [class@Session]
- *
- * Closes the session.
- */
-void
-xdp_session_close (XdpSession *session)
-{
-  g_return_if_fail (XDP_IS_SESSION (session));
-
-  g_dbus_connection_call (session->portal->bus,
-                          PORTAL_BUS_NAME,
-                          session->id,
-                          SESSION_INTERFACE,
-                          "Close",
-                          NULL,
-                          NULL, 0, -1, NULL, NULL, NULL);
-
-  _xdp_session_set_session_state (session, XDP_SESSION_CLOSED);
-  g_signal_emit_by_name (session, "closed");
-}
-
 /**
  * xdp_session_open_pipewire_remote:
  * @session: a [class@Session]
@@ -1021,7 +1041,7 @@ xdp_session_pointer_motion (XdpSession *session,
  *
  * Moves the pointer to a new position in the given streams logical
  * coordinate space.
- * 
+ *
  * May only be called on a remote desktop session
  * with `XDP_DEVICE_POINTER` access.
  */
@@ -1319,3 +1339,114 @@ xdp_session_get_restore_token (XdpSession *session)
 
   return g_strdup (session->restore_token);
 }
+
+/**
+ * xdp_session_get_devices:
+ * @session: a [class@Session]
+ *
+ * Obtains the devices that the user selected.
+ *
+ * Unless the session is active, this function returns `XDP_DEVICE_NONE`.
+ *
+ * Returns: the selected devices
+ */
+XdpDeviceType
+xdp_session_get_devices (XdpSession *session)
+{
+  g_return_val_if_fail (XDP_IS_SESSION (session), XDP_DEVICE_NONE);
+
+  if (session->state != XDP_SESSION_ACTIVE)
+    return XDP_DEVICE_NONE;
+
+  return session->devices;
+}
+
+void
+_xdp_session_set_devices (XdpSession *session,
+                          XdpDeviceType devices)
+{
+  session->devices = devices;
+}
+
+/**
+ * xdp_session_get_streams:
+ * @session: a [class@Session]
+ *
+ * Obtains the streams that the user selected.
+ *
+ * The information in the returned [struct@GLib.Variant] has the format
+ * `a(ua{sv})`. Each item in the array is describing a stream. The first member
+ * is the pipewire node ID, the second is a dictionary of stream properties,
+ * including:
+ *
+ * - position, `(ii)`: a tuple consisting of the position `(x, y)` in the compositor
+ *     coordinate space. Note that the position may not be equivalent to a
+ *     position in a pixel coordinate space. Only available for monitor streams.
+ * - size, `(ii)`: a tuple consisting of (width, height). The size represents the size
+ *     of the stream as it is displayed in the compositor coordinate space.
+ *     Note that this size may not be equivalent to a size in a pixel coordinate
+ *     space. The size may differ from the size of the stream.
+ *
+ * Unless the session is active, this function returns `NULL`.
+ *
+ * Returns: the selected streams
+ */
+GVariant *
+xdp_session_get_streams (XdpSession *session)
+{
+  g_return_val_if_fail (XDP_IS_SESSION (session), NULL);
+
+  if (session->state != XDP_SESSION_ACTIVE)
+    return NULL;
+
+  return session->streams;
+}
+
+void
+_xdp_session_set_streams (XdpSession *session,
+                          GVariant *streams)
+{
+  if (session->streams)
+    g_variant_unref (session->streams);
+  session->streams = streams;
+  if (session->streams)
+    g_variant_ref (session->streams);
+}
+
+/**
+ * xdp_session_get_session_state:
+ * @session: an [class@Session]
+ *
+ * Obtains information about the state of the session that is represented
+ * by @session.
+ *
+ * Returns: the state of @session
+ */
+XdpSessionState
+xdp_session_get_session_state (XdpSession *session)
+{
+  g_return_val_if_fail (XDP_IS_SESSION (session), XDP_SESSION_CLOSED);
+
+  return session->state;
+}
+
+void
+_xdp_session_set_session_state (XdpSession *session,
+                                XdpSessionState state)
+{
+  session->state = state;
+
+  if (state == XDP_SESSION_INITIAL && session->state != XDP_SESSION_INITIAL)
+    {
+      g_warning ("Can't move a session back to initial state");
+      return;
+    }
+  if (session->state == XDP_SESSION_CLOSED && state != XDP_SESSION_CLOSED)
+    {
+      g_warning ("Can't move a session back from closed state");
+      return;
+    }
+
+  if (state == XDP_SESSION_CLOSED)
+    _xdp_session_close (session);
+}
